@@ -21,6 +21,7 @@ from __future__ import absolute_import
 import logging
 import os
 import StringIO
+import subprocess
 import sys
 import time
 import weakref
@@ -38,9 +39,10 @@ import solfege
 from solfege import cfg
 from solfege import filesystem
 from solfege import gu
-from solfege import learningtree as pd
+from solfege import frontpage as pd
 from solfege import lessonfile
 from solfege import lessonfilegui
+from solfege import osutils
 
 class LessonFilePreviewWidget(gtk.VBox):
     def __init__(self, model):
@@ -74,8 +76,6 @@ class LessonFilePreviewWidget(gtk.VBox):
             fn = gu.decode_filename(fn)
             for child in self.g_topic_box.get_children():
                 child.destroy()
-            if fn.startswith(os.getcwdu()):
-                fn = fn[len(os.getcwdu())+1:]
             fn = lessonfile.mk_uri(fn)
             try:
                 self.set_sensitive(True)
@@ -104,6 +104,7 @@ class SelectLessonFileDialog(gtk.FileChooserDialog):
         gtk.FileChooserDialog.__init__(self, _("Select lesson file"),
             parent=parent,
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,))
+        self.set_select_multiple(True)
         pv = LessonFilePreviewWidget(parent.m_model)
         pv.g_ok_button = self.add_button("gtk-ok", gtk.RESPONSE_OK)
         pv.g_ok_button.set_sensitive(False)
@@ -161,17 +162,24 @@ class Section(gtk.VBox):
         hbox.pack_start(button_hbox, False)
         im = gtk.Image()
         im.set_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU)
-        edit_btn = gtk.Button()
-        edit_btn.add(im)
-        edit_btn.connect('clicked', self.on_edit_heading)
-        button_hbox.pack_start(edit_btn, False)
+        button = gtk.Button()
+        button.add(im)
+        button.connect('clicked', self.on_edit_heading)
+        button_hbox.pack_start(button, False)
         #
         im = gtk.Image()
         im.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
-        edit_btn = gtk.Button()
-        edit_btn.add(im)
-        edit_btn.connect('button-release-event', self.on_add)
-        button_hbox.pack_start(edit_btn, False)
+        button = gtk.Button()
+        button.add(im)
+        button.connect('button-release-event', self.on_add)
+        button_hbox.pack_start(button, False)
+        #
+        im = gtk.Image()
+        im.set_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
+        button = gtk.Button()
+        button.add(im)
+        button.connect('button-release-event', self.on_remove)
+        button_hbox.pack_start(button, False)
         #
         im = gtk.Image()
         im.set_from_stock(gtk.STOCK_CUT, gtk.ICON_SIZE_MENU)
@@ -265,6 +273,8 @@ class Section(gtk.VBox):
         menu.append(item)
         menu.show_all()
         menu.popup(None, None, None, event.button, event.time)
+    def on_remove(self, btn, event):
+        self.m_parent.remove_section(self)
     def on_add_link(self, btn):
         if editor_of(self).m_filename:
             open_dir = os.path.split(editor_of(self).m_filename)[0]
@@ -278,20 +288,21 @@ class Section(gtk.VBox):
                 break
             else:
                 assert ret == gtk.RESPONSE_OK
-                fn = gu.decode_filename(dlg.get_filename())
-                assert os.path.isabs(fn)
-                # If the file name is a file in a subdirectory below
-                # lessonfile.exercises_dir in the current working directory,
-                # then the file is a standard lesson file, and it will be
-                # converted to a uri scheme with:
-                fn = lessonfile.mk_uri(fn)
-                # Small test to check that the file actually is a lesson file.
-                try:
-                    lessonfile.infocache.get(fn, 'title')
-                except lessonfile.infocache.FileNotLessonfile:
-                    continue
-                self.m_model.append(fn)
-                self.g_link_box.pack_start(self.create_linkrow(fn), False)
+                for filename in dlg.get_filenames():
+                    fn = gu.decode_filename(filename)
+                    assert os.path.isabs(fn)
+                    # If the file name is a file in a subdirectory below
+                    # lessonfile.exercises_dir in the current working directory,
+                    # then the file is a standard lesson file, and it will be
+                    # converted to a uri scheme with:
+                    fn = lessonfile.mk_uri(fn)
+                    # Small test to check that the file actually is a lesson file.
+                    try:
+                        lessonfile.infocache.get(fn, 'title')
+                    except lessonfile.infocache.FileNotLessonfile:
+                        continue
+                    self.m_model.append(fn)
+                    self.g_link_box.pack_start(self.create_linkrow(fn), False)
                 break
         dlg.destroy()
     def on_add_link_to_new_page(self, menuitem):
@@ -342,6 +353,18 @@ class Section(gtk.VBox):
             item.connect('activate', self.on_edit_linktext, linked)
             item.set_sensitive(bool(not isinstance(linked, basestring)))
             m.append(item)
+            item = gtk.ImageMenuItem(gtk.STOCK_GO_UP)
+            item.connect('activate', self.on_move_up, idx)
+            item.set_sensitive(bool(idx > 0))
+            m.append(item)
+            item = gtk.ImageMenuItem(gtk.STOCK_GO_DOWN)
+            item.connect('activate', self.on_move_down, idx)
+            item.set_sensitive(bool(idx < len(self.m_model) - 1))
+            m.append(item)
+            item = gtk.ImageMenuItem(gtk.STOCK_EDIT)
+            item.set_sensitive(isinstance(linked, unicode))
+            item.connect('activate', self.on_edit_file, idx)
+            m.append(item)
             m.show_all()
             m.popup(None, None, None, event.button, event.time)
             return True
@@ -380,6 +403,15 @@ class Section(gtk.VBox):
                 entry.destroy()
                 return True
         entry.connect('key-release-event', keyup)
+    def on_edit_file(self, menuitem, linked):
+        try:
+            try:
+                subprocess.call((cfg.get_string("programs/text-editor"),
+                             lessonfile.uri_expand(self.m_model[linked])))
+            except OSError, e:
+                 raise osutils.BinaryForProgramException("Text editor", cfg.get_string("programs/text-editor"), e)
+        except osutils.BinaryForProgramException, e:
+            solfege.win.display_error_message2(e.msg1, e.msg2)
     def on_cut(self, btn):
         self.m_parent.cut_section(self)
     def on_cut_link(self, menuitem, idx):
@@ -401,6 +433,19 @@ class Section(gtk.VBox):
             row = self.create_linkrow(mobj)
             self.g_link_box.pack_start(row)
             self.g_link_box.reorder_child(row, idx)
+    def on_move_up(self, btn, idx):
+        """
+        Move the link one row up.
+        """
+        assert idx > 0
+        self.m_model[idx], self.m_model[idx - 1] = self.m_model[idx - 1], self.m_model[idx]
+        self.g_link_box.reorder_child(self.g_link_box.get_children()[idx], idx - 1)
+    def on_move_down(self, btn, idx):
+        """
+        Move the link one row down.
+        """
+        self.m_model[idx], self.m_model[idx + 1] = self.m_model[idx + 1], self.m_model[idx]
+        self.g_link_box.reorder_child(self.g_link_box.get_children()[idx], idx + 1)
 
 
 class Column(gtk.VBox):
@@ -433,6 +478,10 @@ class Column(gtk.VBox):
         Editor.clipboard.append(self.m_model[idx])
         del self.m_model[idx]
         self.g_section_box.get_children()[idx].destroy()
+    def remove_section(self, section):
+        idx = self.g_section_box.get_children().index(section)
+        del self.m_model[idx]
+        self.g_section_box.get_children()[idx].destroy()
     def on_add_section(self, btn):
         # We write "Untitled%s" % "" instead of just "Untitled" here
         # since "Untitled%s" is already translated in many languages.
@@ -556,47 +605,15 @@ class Clipboard(list):
         list.append(self, obj)
         self.update_buttons()
     def register_paste_button(self, button, accepts_types):
+        button.set_sensitive(bool(self) and isinstance(self[-1], types))
         self.m_paste_buttons.append((button, accepts_types))
     def update_buttons(self):
         for button, types in self.m_paste_buttons:
             button.set_sensitive(bool(self) and isinstance(self[-1], types))
 
-class PropertiesDialog(gtk.Dialog):
-    def __init__(self, model, parent):
-        gtk.Dialog.__init__(self, "Properties", parent,
-            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
-                     gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-        vbox = gu.hig_dlg_vbox()
-        self.vbox.pack_start(vbox)
-        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        #
-        label = gtk.Label()
-        label.set_alignment(1.0, 0.5)
-        sizegroup.add_widget(label)
-        label.set_markup(u"<b>%s</b>" % _("Filename:"))
-        label2 = gtk.Label(parent.m_filename)
-        label2.set_alignment(0.0, 0.5)
-        hbox = gtk.HBox()
-        hbox.set_spacing(gu.hig.SPACE_MEDIUM)
-        hbox.pack_start(label, False)
-        hbox.pack_start(label2, True, True)
-        vbox.pack_start(hbox, False)
-        #
-        label = gtk.Label()
-        label.set_alignment(1.0, 0.5)
-        label.set_markup(u"<b>%s</b>" % _("Front page title:"))
-        sizegroup.add_widget(label)
-        self.g_title_entry = gtk.Entry()
-        self.g_title_entry.set_text(model.m_name)
-        hbox = gtk.HBox()
-        hbox.set_spacing(gu.hig.SPACE_MEDIUM)
-        hbox.pack_start(label, False)
-        hbox.pack_start(self.g_title_entry, True, True)
-        vbox.pack_start(hbox, False)
-
 
 class Editor(gtk.Window, gu.EditorDialogBase):
-    savedir = os.path.join(filesystem.user_data(), 'learningtrees')
+    savedir = os.path.join(filesystem.user_data(), u'exercises', u'user')
     # The clipboard will be shared between all Editor instances
     clipboard = Clipboard()
     def __init__(self, filename=None):
@@ -608,9 +625,17 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         self.add(self.g_main_box)
         self.g_actiongroup.add_actions([
             ('GoBack', gtk.STOCK_GO_BACK, None, None, None, self.go_back),
-            ('Properties', gtk.STOCK_PROPERTIES, None, None, None, self.properties),
         ])
         self.setup_toolbar()
+        self.g_title_hbox = gtk.HBox()
+        self.g_title_hbox.set_spacing(gu.hig.SPACE_SMALL)
+        self.g_title_hbox.set_border_width(gu.hig.SPACE_SMALL)
+        label = gtk.Label()
+        label.set_markup(u"<b>%s</b>" % _("Front page title:"))
+        self.g_title_hbox.pack_start(label, False)
+        self.g_fptitle = gtk.Entry()
+        self.g_title_hbox.pack_start(self.g_fptitle)
+        self.g_main_box.pack_start(self.g_title_hbox, False)
         # This dict maps the windows created for all pages belonging to
         # the file.
         self.m_page_mapping = {}
@@ -621,12 +646,12 @@ class Editor(gtk.Window, gu.EditorDialogBase):
             self.m_model = pd.Page(_("Untitled%s") % self.m_instance_number,
                     pd.Column())
             self.set_not_modified()
-        self.m_page_mapping[id(self.m_model)] = Page(self.m_model, self)
-        self.g_visible_page = self.m_page_mapping[id(self.m_model)]
-        self.g_main_box.pack_start(self.m_page_mapping[id(self.m_model)])
+        self.add_page(Page(self.m_model, self))
         self.clipboard.update_buttons()
         self.show_all()
         self.add_to_instance_dict()
+        self.g_fptitle.set_text(self.m_model.m_name)
+        self.g_fptitle.connect('changed', self.on_frontpage_title_changed)
     def __del__(self):
         logging.debug("fpeditor.Editor.__del__, filename=" % self.m_filename)
     def add_page(self, page):
@@ -642,20 +667,22 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         """
         Hide the currently visible page, and show PAGE instead.
         """
-        self.g_visible_page.hide()
+        try:
+            self.g_visible_page.hide()
+        except AttributeError:
+            pass
         self.g_visible_page = page
         page.show()
+        if isinstance(page.m_parent, Page):
+            self.g_title_hbox.hide()
+        else:
+            self.g_title_hbox.show()
         self.g_ui_manager.get_widget("/Toolbar/GoBack").set_sensitive(
             not isinstance(self.g_visible_page.m_parent, Editor))
     def go_back(self, *action):
         self.show_page(self.g_visible_page.m_parent)
-    def properties(self, *action):
-        dlg = PropertiesDialog(self.m_model, self)
-        dlg.show_all()
-        ret = dlg.run()
-        if ret == gtk.RESPONSE_ACCEPT:
-            self.m_model.m_name = dlg.g_title_entry.get_text()
-        dlg.destroy()
+    def on_frontpage_title_changed(self, widget):
+        self.m_model.m_name = widget.get_text()
     def setup_toolbar(self):
         self.g_ui_manager.insert_action_group(self.g_actiongroup, 0)
         uixml = """
@@ -666,7 +693,6 @@ class Editor(gtk.Window, gu.EditorDialogBase):
           <toolitem action='Open'/>
           <toolitem action='Save'/>
           <toolitem action='SaveAs'/>
-          <toolitem action='Properties'/>
           <toolitem action='Close'/>
           <toolitem action='Help'/>
          </toolbar>
@@ -699,10 +725,10 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         else:
             try:
                 win = Editor(fn)
+                win.show()
             except IOError, e:
                 gu.dialog_ok(_("Loading file '%(filename)s' failed: %(msg)s") %
                         {'filename': fn, 'msg': str(e).decode('utf8', 'replace')})
-            win.show()
     def load_file(self, filename):
         """
         Load a file into a empty, newly created Editor object.
@@ -747,9 +773,24 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         self.set_not_modified()
         # We do test for solfege.win since it is not available during testing
         if getattr(solfege, 'win', None):
-            solfege.win.on_learning_tree_changed()
+            solfege.win.on_frontpage_changed()
     def on_show_help(self, *w):
         return
+    def get_save_as_dialog(self):
+        dialog = gu.EditorDialogBase.get_save_as_dialog(self)
+        ev2 = gtk.EventBox()
+        ev2.set_name("DIALOGWARNING2")
+        ev = gtk.EventBox()
+        ev.set_border_width(gu.hig.SPACE_SMALL)
+        ev2.add(ev)
+        ev.set_name("DIALOGWARNING")
+        label = gtk.Label()
+        label.set_padding(gu.hig.SPACE_MEDIUM, gu.hig.SPACE_MEDIUM)
+        ev.add(label)
+        label.set_markup(_("<b>IMPORTANT:</b> Your front page file <b>must</b> be saved in a subdirectory below the directory named exercises. See the user manual for details."))
+        dialog.set_extra_widget(ev2)
+        ev2.show_all()
+        return dialog
 
 if __name__ == '__main__':
     gtk.link_button_set_uri_hook(lambda a, b: None)
